@@ -12,5 +12,7 @@ define('USS_ERR_004', "ユーザ名またはパスワードが間違っていま
 define('USS_ERR_005', "パスワードに使用している文字が不正です。");
 define('USS_ERR_006', "パスワードが短すぎます。");
 define('USS_ERR_007', "パスワードが長すぎます。");
-define('USS_ERR_101', "シークレットの設定に失敗しました。");
+define('USS_ERR_101', "シークレットが不正な値です。");
+define('USS_ERR_102', "シークレットの設定に失敗しました。");
+define('USS_ERR_103', "PINが不正な値です。");
 ?>
