@@ -73,13 +73,15 @@ class Lism_AppObject extends Ethna_AppObject
             $this->bindpw = $conf['bindpw'];
         }
 
-        $ldap = $this->connect();
-        if (Ethna::isError($ldap)) {
-            return $ldap;
-        }
-        $this->my_ldap = $ldap;
+        if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
 
-        register_shutdown_function(array(&$this, 'disconnect'));
+            register_shutdown_function(array(&$this, 'disconnect'));
+        }
 
         if (is_null($this->prop_def)) {
             $this->prop_def = $this->_getPropDef();
@@ -102,6 +104,21 @@ class Lism_AppObject extends Ethna_AppObject
 
         if (is_null($this->id)) {
             $this->id = $this->_getDn();
+        } else {
+            if (is_null($this->object_def)) {
+                return null;
+            }
+            foreach ($this->object_def as $object_name => $object_attr) {
+                // use 1st one
+                break;
+            }
+
+            $conf = $this->config->get('secioss');
+
+            // オブジェクトのプライマリキー定義構築
+            if (is_null($this->id_def)) {
+                $this->id_def = $conf[$object_name]['id'];
+            }
         }
 
         return 0;
@@ -129,7 +146,10 @@ class Lism_AppObject extends Ethna_AppObject
             return $this->prop[$key];
         }
 
-        return parent::get($key);
+        if (isset($this->prop[$key])) {
+            return $this->prop[$key];
+        }
+        return null;
     }
     // }}}
 
@@ -179,10 +199,20 @@ class Lism_AppObject extends Ethna_AppObject
             $info[$key] = $value;
         }
 
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
         $r = @ldap_add($this->my_ldap, $this->id, $info);
         if (!$r) {
             return Ethna::raiseError(ldap_error($this->my_ldap));
         }
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
 
         // バックアップ/キャッシュ更新
         $this->prop_backup = $this->prop;
@@ -203,16 +233,34 @@ class Lism_AppObject extends Ethna_AppObject
     {
         $info = array();
         foreach ($this->prop as $key => $value) {
-            if ($key == 'area' || $key == 'unit' || $key == 'category' || $key == 'system' || $key == 'parent' || $key == 'basedn') {
+            if ($key == $this->id_def || $key == 'area' || $key == 'unit' || $key == 'category' || $key == 'system' || $key == 'basedn') {
                 continue;
             }
-            $info[$key] = $value;
+            if ($key == 'parent') {
+                $info['lismparentdn'] = $value;
+            } else {
+                $info[$key] = $value;
+            }
         }
 
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
         $r = @ldap_modify($this->my_ldap, $this->id, $info);
         if (!$r) {
             return Ethna::raiseError(ldap_error($this->my_ldap));
         }
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
+
+        if (isset($this->prop['parent'])) {
+            $this->id = preg_replace('/^([^,]+,).+$/', '$1', $this->id).$this->prop['parent'];
+        }
 
         // バックアップ/キャッシュ更新
         $this->prop_backup = $this->prop;
@@ -247,10 +295,20 @@ class Lism_AppObject extends Ethna_AppObject
      */
     function remove()
     {
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
         $r = @ldap_delete($this->my_ldap, $this->id);
         if (!$r) {
             return Ethna::raiseError(ldap_error($this->my_ldap));
         }
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
 
         // プロパティ/バックアップ/キャッシュクリア
         $this->id = $this->prop = $this->prop_backup = null;
@@ -275,9 +333,9 @@ class Lism_AppObject extends Ethna_AppObject
      */
     function searchId($filter = null, $order = null, $offset = null, $count = null)
     {
-        $r = $this->_getLDAP_Search($filter, $order, $offset, $count);
+        $r = $this->_getLDAP_Search(null, $filter, $order, $offset, $count);
         if (Ethna::isError($r)) {
-            return $r;
+            return array(0);
         }
 
         $id_list = array();
@@ -306,9 +364,9 @@ class Lism_AppObject extends Ethna_AppObject
     function searchProp($keys = null, $filter = null, $order = null,
                         $offset = null, $count = null)
     {
-        $r = $this->_getLDAP_Search($filter, $order, $offset, $count);
-        if (Ethna::isError($r)) {
-            return $r;
+        $r = $this->_getLDAP_Search($keys, $filter, $order, $offset, $count);
+        if (Ethna::isError($r) || !is_array($r[1])) {
+            return array(0);
         }
 
         $prop_list = array();
@@ -541,7 +599,7 @@ class Lism_AppObject extends Ethna_AppObject
         }
 
         // プロパティ取得
-        $r = $this->_getLDAP_Search($filter, null, null, null);
+        $r = $this->_getLDAP_Search(null, $filter, null, null, null);
         if (Ethna::isError($r)) {
             return;
         }
@@ -574,12 +632,12 @@ class Lism_AppObject extends Ethna_AppObject
     function _setPropByValue($prop)
     {
         $def = $this->getDef();
-        if (is_null($def)) {
+        if (is_null($def) || !count($def)) {
             $def = $prop;
         }
 
         foreach ($def as $key => $value) {
-            if ($key != 'id' && isset($prop[$key])) {
+            if ($key != 'id' && array_key_exists($key, $prop)) {
                 $this->prop[$key] = $prop[$key];
             }
         }
@@ -620,12 +678,18 @@ class Lism_AppObject extends Ethna_AppObject
             return Ethna::raiseError("ObjectClass not defined");
         }
 
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
         $r = @ldap_read($this->my_ldap, '', 'objectClass=*', array('subschemaSubEntry'));
         if (ldap_errno($this->my_ldap) == 32) {
             // for PHP 4.x
             $r = @ldap_read($this->my_ldap, 'cn=Subschema', 'objectClass=*', array('subschemaSubEntry'));
         }
-
         if (!$r) {
             return null;
         }
@@ -634,7 +698,17 @@ class Lism_AppObject extends Ethna_AppObject
         if ($entries['count'] != 1) {
             return Ethna::raiseError("Can't get subschemaSubEntry");
         }
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
 
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
         $r = @ldap_read($this->my_ldap, $entries[0]['subschemasubentry'][0], 'objectClass=*', array('objectClasses'));
         if (!$r) {
             return Ethna::raiseError(ldap_error($this->my_ldap));
@@ -645,6 +719,10 @@ class Lism_AppObject extends Ethna_AppObject
             return Ethna::raiseError("Can't get objectClasses");
         }
 
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
+
         $def = array();
         foreach ($this->object_def[$object]['objectclass'] as $objectclass) {
             foreach ($entries[0]['objectclasses'] as $oc_def) {
@@ -672,13 +750,34 @@ class Lism_AppObject extends Ethna_AppObject
      *                  1 => $offset, $countにより指定された件数のオブジェクト一覧)
      *                  Ethna_Error:エラー
      */
-    function _getLDAP_Search($filter, $order, $offset, $count)
+    function _getLDAP_Search($keys, $filter, $order, $offset, $count)
     {
+        $conf = $this->config->get('ldap');
+        $sorted = false;
+
         $basedn = $this->_getLDAP_SearchBase($filter);
+        $attrs = is_array($keys) ? $keys : array();
 
         //LDAP用のフィルタに変換
         $ldap_filter = $this->_getLDAP_SearchFilter($filter);
 
+        if (is_array($order)) {
+            foreach ($order as $k => $v) {
+                break;
+            }
+        } else {
+            $k = null;
+            $v = -1;
+        }
+
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $ldap = $this->connect();
+            if (Ethna::isError($ldap)) {
+                return $ldap;
+            }
+            $this->my_ldap = $ldap;
+        }
+
         if (!$this->my_ldap) {
             return Ethna::raiseError("Could not connect to LDAP server.", E_DB_CONNECT);
         }
@@ -686,7 +785,29 @@ class Lism_AppObject extends Ethna_AppObject
         if (isset($filter['id'])) {
             $r = @ldap_read($this->my_ldap, $basedn, $ldap_filter);
         } else {
-            $r = @ldap_search($this->my_ldap, $basedn, $ldap_filter);
+            if (isset($conf['option']) && ($offset || $count || $k)) {
+                if (!$k) {
+                    $k = $this->id_def;
+                }
+                $control = '';
+                if (preg_match('/,vlv,/', ','.$conf['option'].',')) {
+                    if (!$k) {
+                        $k = $this->id_def;
+                    }
+                    $control = "vlv=$count,$offset&sort=$k:2.5.13.3";
+                }
+                if (preg_match('/,paged,/', ','.$conf['option'].',')) {
+                    $control = "paged=$offset,$count";
+                }
+                if (preg_match('/,sort,/', ','.$conf['option'].',')) {
+                    $control = ($control ? $control.'&' : '')."sort=$k:2.5.13.3";
+                }
+                if ($control) {
+                    $sorted = true;
+                    $ldap_filter = "(&(lismControl=$control)$ldap_filter)";
+                }
+            }
+            $r = @ldap_search($this->my_ldap, $basedn, $ldap_filter, $attrs);
         }
 
         if ($r) {
@@ -698,13 +819,16 @@ class Lism_AppObject extends Ethna_AppObject
         }
         ldap_free_result($r);
 
-        if (is_array($order)) {
-            foreach ($order as $k => $v) {
-                break;
+        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
+            $this->disconnect();
+        }
+
+        if ($sorted) {
+            $list = array();
+            for ($i = 0; $i < $entries['count']; $i++) {
+                array_unshift($list, $entries[$i]);
             }
-        } else {
-            $k = null;
-            $v = -1;
+            return array($entries['count'], $list);
         }
 
         list($object_name) = array_keys($this->object_def);
@@ -777,8 +901,8 @@ class Lism_AppObject extends Ethna_AppObject
             $basedn = $conf['basedn'];
         }
 
-        if (isset($conf['system']) && isset($filter['system'])) {
-            $basedn = 'ou=' . $filter['system'] . ',' . $basedn;
+        if (isset($filter['system'])) {
+            $basedn = 'ou=' . $filter['system'] . ',' . (isset($conf['suffix']) ? $conf['suffix'] : $basedn);
         }
 
         if (isset($conf['area']['id']) && isset($filter['area'])) {
@@ -830,12 +954,19 @@ class Lism_AppObject extends Ethna_AppObject
             return null;
         }
 
+        list($object_name) = array_keys($this->object_def);
+        $conf = $this->config->get('secioss');
+
         $ldap_filter = '';
-        foreach ($object_attr['objectclass'] as $objectclass) {
-            if ($ldap_filter) {
-                $ldap_filter = "(&$ldap_filter(objectClass=$objectclass))";
-            } else {
-                $ldap_filter = "(objectClass=$objectclass)";
+        if (isset($conf[$object_name]['filter'])) {
+            $ldap_filter = $conf[$object_name]['filter'];
+        } else {
+            foreach ($object_attr['objectclass'] as $objectclass) {
+                if ($ldap_filter) {
+                    $ldap_filter = "(&$ldap_filter(objectClass=$objectclass))";
+                } else {
+                    $ldap_filter = "(objectClass=$objectclass)";
+                }
             }
         }
 
@@ -845,19 +976,25 @@ class Lism_AppObject extends Ethna_AppObject
 
         foreach ($filter as $key => $values){
             $key = strtolower($key);
+            $tmp_filter = '';
+            foreach (preg_split('/\|/', $key) as $attr) {
+                if ($attr == 'id' || (!is_null($this->prop_def) && isset($this->prop_def[preg_replace('/;.*/', '', $attr)]) == false)) {
+                    continue;
+                }
 
-            if ($key == 'id' || (!is_null($this->prop_def) && isset($this->prop_def[$key]) == false)) {
-                continue;
+                if (is_array($values)) {
+                    foreach ($values as $value) {
+                        list($value, $type) = $this->_parseLdapValue($value);
+                        $tmp_filter = $tmp_filter ? "(|$tmp_filter($attr$type$value))" : "($attr$type$value)";
+                    }
+                } else {
+                    list($values, $type) = $this->_parseLdapValue($values);
+                    $tmp_filter = $tmp_filter ? "(|$tmp_filter($attr$type$values))" : "($attr$type$values)";
+                }
             }
 
-            if (is_array($values)) {
-                foreach ($values as $value) {
-                    $value = addcslashes($value,'()');
-                    $ldap_filter = "(&$ldap_filter($key=$value))";
-                }
-            } else {
-                $values = addcslashes($values,'()');
-                $ldap_filter = "(&$ldap_filter($key=$values))";
+            if ($tmp_filter) {
+                $ldap_filter = "(&$ldap_filter$tmp_filter)";
             }
         }
 
@@ -865,6 +1002,23 @@ class Lism_AppObject extends Ethna_AppObject
     }
     // }}}
 
+    function _parseLdapValue($value)
+    {
+        $fchr = substr($value, 0, 1);
+        if ($fchr == '>') {
+            $type = '>=';
+            $value = substr($value, 1);
+        } else if ($fchr == '<') {
+            $type = '<=';
+            $value = substr($value, 1);
+        } else {
+            $type = '=';
+        }
+        $value = addcslashes($value,'()');
+
+        return array($value, $type);
+    }
+
     // {{{ _getDn
     /**
      *  オブジェクトDNを返す
@@ -942,7 +1096,7 @@ class Lism_AppObject extends Ethna_AppObject
             $key = $entry[$i];
 
             if ($entry[$key]['count'] == 1) {
-                if ($entry[$key][0] && !preg_match('/^ *$/', $entry[$key][0])) {
+                if ($entry[$key][0] !== '' && $entry[$key][0] !== ' ') {
                     $row[$key] = $entry[$key][0];
                 }
             } else {
@@ -962,13 +1116,15 @@ class Lism_AppObject extends Ethna_AppObject
 
         $basedn = $conf['basedn'];
         $dn = strtolower($entry['dn']);
-        if (isset($conf['system']) &&
-          preg_match('/ou='. '([^,]*),' .$basedn . '$/i', $dn, $match)) {
+        if (isset($conf['suffix']) &&
+          preg_match('/ou='. '([^,]*),' .$conf['suffix'] . '$/i', $dn, $match)) {
             $row['system'] = $match[1];
-            $basedn = 'ou=' . $match[1] . ',' . $basedn;
+            if ($basedn == $conf['suffix']) {
+                $basedn = 'ou=' . $match[1] . ',' . $basedn;
+            }
         }
 
-        if (isset($conf['area']['id']) && 
+        if (isset($conf['area']) && 
           preg_match('/' . $conf['area']['id'] . '=([^,]*),' . $basedn . '$/i', $dn, $match)) {
             $row['area'] = $match[1];
             $basedn = $conf['area']['id'] . '=' . $match[1] . ',' . $basedn;
@@ -978,8 +1134,8 @@ class Lism_AppObject extends Ethna_AppObject
             $basedn = $conf[$object_name]['objectdn'] . ',' . $basedn;
         }
 
-        if (isset($conf[$object_name]['categoryid']) &&
-           preg_match('/' . $conf[$object_name]['categoryid'] . '=([^,]*),' . $basedn . '$/i', $dn, $match)) {
+        if (isset($conf['category']) &&
+           preg_match('/' . $conf[$object_name]['id'] . '=[^,]+,' . $conf['category']['id'] . '=([^,]*),' . $basedn . '$/i', $dn, $match)) {
             $row['category'] = $match[1];
         }
 
