@@ -8,7 +8,7 @@
  */
 
 if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN') {
-    define('HTPASSWD', '/usr/bin/htpasswd -b');
+    define('HTPASSWD', '/usr/bin/htpasswd');
 }
 
 /**
@@ -49,7 +49,8 @@ class Lism_ConfigManager extends Ethna_AppManager
                 $values[] = preg_match("/$key += +\"?([^\"\n]+)\"?/", $contents, $matches) ? $matches[1] / 60 : 0;
                 break;
               case 'pwallowtype':
-                $regexp = preg_match("/pwdeny += +\"?([^\"\n]+)\"?/", $contents, $matches) ? $matches[1] : '';
+                $regexp = preg_match("/pwdeny += +\"?([^\n]+)\"?/", $contents, $matches) ? $matches[1] : '';
+                $regexp = preg_replace('/"$/', '', $regexp);
                 if ($regexp == '[^a-zA-Z0-9]') {
                     $values[] = 'alphnum';
                 } else if (preg_match('/\[\^a-zA-Z0-9[^\]]+\]/', $regexp)) {
@@ -59,15 +60,20 @@ class Lism_ConfigManager extends Ethna_AppManager
                 }
                 break;
               case 'pwallowchars':
-                $regexp = preg_match("/pwdeny += +\"?([^\"\n]+)\"?/", $contents, $matches) ? $matches[1] : '';
+                $regexp = preg_match("/pwdeny += +\"?([^\n]+)\"?/", $contents, $matches) ? $matches[1] : '';
+                $regexp = preg_replace('/"$/', '', $regexp);
                 if ($regexp == '[^a-zA-Z0-9]') {
                     $values[] = '';
-                } else if (preg_match('/\[\^a-zA-Z0-9([^\]]+)\]/', $regexp, $matches)) {
-                    $values[] = $matches[1];
+                } else if (preg_match('/\[\^a-zA-Z0-9(.+)\]$/', $regexp, $matches)) {
+                    $values[] = preg_replace(array('/\\\([^\\\]{1})/', '/\\\\\\\/'), array('\1', '\\'), $matches[1]);
                 } else {
                     $values[] = '';
                 }
                 break;
+              case 'pwallowrule':
+                $regexp = preg_match("/pwallow\[\] += \"?([^\n]+)\"?/", $contents, $matches) ? $mactches[1] : '';
+                $values[] = $regexp ? true : false;
+                break;
               case 'pwinhistory':
                 $values[] = preg_match("/$key += +\"?([^\"\n]+)\"?/", $contents, $matches) ? $matches[1] : 0;
                 break;
@@ -198,6 +204,16 @@ class Lism_ConfigManager extends Ethna_AppManager
                                break;
                             }
                             break;
+                          case 'pwallow':
+                            if (!array_key_exists('pwallowrule', $params)) {
+                                continue;
+                            }
+                            if ($params['pwallowrule']) {
+                                $value = array('[a-z]', '[A-Z]', '[0-9]');
+                            } else {
+                                $value = '';
+                            }
+                            break;
                           case 'uri':
                           case 'binddn':
                           case 'bindpw':
@@ -221,7 +237,13 @@ class Lism_ConfigManager extends Ethna_AppManager
                             break;
                         }
                     }
-                    fwrite($fp, "$key2 = \"$value\"\n");
+                    if (is_array($value)) {
+                        for ($i = 0; $i < count($value); $i++) {
+                            fwrite($fp, $key2.'[] = "'.$value[$i]."\"\n");
+                        }
+                    } else {
+                        fwrite($fp, "$key2 = \"$value\"\n");
+                    }
                 }
                 fwrite($fp, "\n");
             } else {
@@ -309,17 +331,19 @@ class Lism_ConfigManager extends Ethna_AppManager
                 }
 
                 if ($sync_flag) {
-                    $sync_cluster[] = $sdata->attributes()->name;
+                    $sync_cluster[] = (string)$sdata->attributes()->name;
                 }
                 if ($master_flag) {
-                    $sync_master[] = $sdata->attributes()->name;
+                    $sync_master[] = (string)$sdata->attributes()->name;
                 }
             }
             if (array_search('sync_cluster', $params) !== false) {
                 $values[] = $sync_cluster;
+                unset($params[array_search('sync_cluster', $params)]);
             }
             if (array_search('sync_master', $params) !== false) {
                 $values[] = $sync_master;
+                unset($params[array_search('sync_master', $params)]);
             }
         }
 
@@ -447,6 +471,7 @@ class Lism_ConfigManager extends Ethna_AppManager
                         }
                         break;
                       case 'GoogleApps':
+                      case 'Salesforce':
                         switch($key) {
                           default:
                             $values[] = $storage->$key;
@@ -466,7 +491,7 @@ class Lism_ConfigManager extends Ethna_AppManager
     function setXmlFile($file, $params, $dname = 'LDAP') {
         $output = BASEDIR.'/tmp/'.basename($file);
         $orgdname = $dname;
-        $dname = preg_replace("/\./", "_", $dname);
+        $dname = preg_replace("/[^0-9a-zA-Z_]/", "_", $dname);
 
         $lp = $this->_lock($file, LOCK_EX);
 
@@ -477,6 +502,7 @@ class Lism_ConfigManager extends Ethna_AppManager
             return Ethna::raiseError("Can't open $file");
         }
         $dom->formatOutput = true;
+        $dom->encoding = 'UTF-8';
         $delement = $dom->documentElement;
 
         $sync = $dom->getElementsByTagName("sync")->item(0);
@@ -534,7 +560,7 @@ class Lism_ConfigManager extends Ethna_AppManager
 
             switch ($sname) {
               case 'SOAP':
-                $data->getElementByTagName("container")->item[0]->getElementByTagName("rdn")->item[0]->nodeValue = "ou=$orgdname";
+                $data->getElementsByTagName("container")->item(0)->getElementsByTagName("rdn")->item(0)->nodeValue = "ou=$orgdname";
                 break;
               default:
             }
@@ -557,7 +583,6 @@ class Lism_ConfigManager extends Ethna_AppManager
             $data->parentNode->removeChild($data);
 
             $dom->save(BASEDIR.'/tmp/'.basename($file));
-            $dom->saveXML();
 
             if (!move($output, $file)) {
                 $this->_unlock($lp);
@@ -709,6 +734,7 @@ class Lism_ConfigManager extends Ethna_AppManager
                 }
                 break;
               case 'GoogleApps':
+              case 'Salesforce':
                 switch($key) {
                   default:
                     $storage->getElementsByTagName($key)->item(0)->nodeValue = $params[$key];
@@ -719,7 +745,6 @@ class Lism_ConfigManager extends Ethna_AppManager
         }
 
         $dom->save($output);
-        $dom->saveXML();
 
         if (!move($output, $file)) {
             $this->_unlock($lp);
@@ -806,11 +831,7 @@ class Lism_ConfigManager extends Ethna_AppManager
         return $rewrite;
     }
 
-    function setPasswdFile($file, $id, $password) {
-        if (!$password) {
-            return Ethna::raiseError("Password is empty");
-        }
-
+    function setPasswdFile($file, $id, $password = null) {
         if (!(file_exists($file))) {
             return Ethna::raiseError("$file doesn't exist");
         }
@@ -818,7 +839,11 @@ class Lism_ConfigManager extends Ethna_AppManager
         $lp = $this->_lock($file, LOCK_EX);
 
         if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN') {
-            system(HTPASSWD." $file $id $password", $rc);
+            if ($password) {
+                system(HTPASSWD." -b $file $id $password", $rc);
+            } else {
+                system(HTPASSWD." -D $file $id", $rc);
+            }
             if ($rc !== 0) {
                 return Ethna::raiseError("htpasswd failed");
             }
