@@ -221,7 +221,7 @@ class Secioss_Action_SystemCreate extends Secioss_ActionClass
 
             return 'System_Create';
         }
-        if (system(LISMRELOAD) == "0") {
+        if (system(LISMRELOAD) === false) {
             $logger->log(LOG_ERR,$r->getMessage());
             $this->ae->add(null, C_RELOAD_ERR_MSG);
 
