@@ -130,7 +130,7 @@ class Secioss_Action_SystemDelete extends Secioss_ActionClass
             }
         }
 
-        if (system(LISMRELOAD) == "0") {
+        if (system(LISMRELOAD) === false) {
             $logger->log(LOG_ERR,$r->getMessage());
             $error = true;
         }
