@@ -141,7 +141,7 @@ class Secioss_Action_SystemUpdate extends Secioss_ActionClass
 
                 return 'System_Update';
             }
-            if (system(LISMRELOAD) == "0") {
+            if (system(LISMRELOAD) === false) {
                 $logger->log(LOG_ERR,$r->getMessage());
                 $this->ae->add(null, _et(C_RELOAD_ERR_MSG));
 
