@@ -3,21 +3,21 @@
  *  {$view_path}
  *
  *  @author     {$author}
- *  @package    Serveradmin
- *  @version    $Id: skel.view.php 387 2006-11-06 14:31:24Z cocoitiban $
+ *  @package    Secioss
+ *  @version    $Id$
  */
 
 /**
- *  {$forward_name}ӥ塼μ
+ *  {$forward_name} view implementation.
  *
  *  @author     {$author}
  *  @access     public
- *  @package    Serveradmin
+ *  @package    Secioss
  */
-class {$view_class} extends Serveradmin_ViewClass
+class {$view_class} extends Secioss_ViewClass
 {
     /**
-     *  
+     *  preprocess before forwarding.
      *
      *  @access public
      */
@@ -25,4 +25,5 @@ class {$view_class} extends Serveradmin_ViewClass
     {
     }
 }
+
 ?>
