@@ -36,15 +36,11 @@ function doAssignGroup(action, group_list, message) {
     //
     if (action == 'action_user_assignGroup' || action == 'action_resource_assignGroup') {
         //
-        if (!confirm(
-            message + "\n"
-            + "よろしいですか？")) {
+        if (!confirm(message)) {
             return;
         }
     } else if (action == 'action_user_deassignGroup' || action == 'action_resource_deassignGroup') {
-        if (!confirm(
-            message + "\n"
-            + "よろしいですか？")) {
+        if (!confirm(message)) {
             return;
         }
     }
