@@ -1,8 +1,4 @@
 //
-function goBack(action) {
-    top.menu.doSearch(action, 1, 0);
-}
-//
 function goAction(action) {
     //
     var form = document.input;
@@ -13,6 +9,8 @@ function goAction(action) {
     form.elements[action].value = "true";
     //
     form.submit();
+    //
+    form.elements[action].value = "";
 }
 //
 function doClearForm(key) {
