@@ -1,14 +1,12 @@
 //
-function doCreate(action, name) {
+function doCreate(action, message) {
     //
     var form = document.input;
     if (form == null) {
         return;
     }
     //
-    if (!confirm(
-        name + "を保存します。\n"
-        + "よろしいですか？")) {
+    if (!confirm(message)) {
         return;
     }
     //
